@@ -23,6 +23,7 @@
 #include <stdlib.h>
 #include <stdbool.h>
 #include <string.h>
+#include <stdarg.h>
 #include <stdint.h>
 #include <inttypes.h>
 #include <signal.h>
@@ -90,6 +91,9 @@
 #define OPTIMIZE3
 #endif
 
+#define FLOAT_TINY		(0.0000001)
+#define FLOAT_CMP(a, b)		(fabs(a - b) < FLOAT_TINY)
+
 typedef struct timer_info {
 	struct timer_info *next;	/* Next in list */
 	struct timer_info *hash_next;	/* Next in hash list */
@@ -142,6 +146,8 @@ typedef struct {
 static const char *app_name = "eventstat";
 static const char *proc_timer_stats = "/proc/timer_stats";
 
+static void es_printf(const char *fmt, ...) __attribute__((format(printf, 1, 2)));
+
 static timer_stat_t *timer_stat_free_list; /* free list of timer stats */
 static timer_info_t *timer_info_list;	/* cache list of timer_info */
 static timer_info_t *timer_info_hash[TABLE_SIZE]; /* hash of timer_info */
@@ -467,7 +473,7 @@ static void sample_add(
 		return;
 
 	for (sdl = sample_delta_list; sdl; sdl = sdl->next) {
-		if (sdl->whence == whence) {
+		if (FLOAT_CMP(sdl->whence, whence)) {
 			found = true;
 			break;
 		}
@@ -519,8 +525,8 @@ inline HOT static sample_delta_item_t *sample_find(
  */
 static int info_compare_total(const void *item1, const void *item2)
 {
-	timer_info_t **info1 = (timer_info_t **)item1;
-	timer_info_t **info2 = (timer_info_t **)item2;
+	timer_info_t *const *info1 = (timer_info_t *const *)item1;
+	timer_info_t *const *info2 = (timer_info_t *const *)item2;
 
 	if ((*info2)->total == (*info1)->total)
 		return 0;
@@ -661,7 +667,6 @@ static void samples_dump(const char *filename)
 	FILE *fp;
 	uint64_t count = 0;
 	double first_time = -1.0;
-	double duration;
 	timer_info_t *info;
 	sample_delta_list_t *sdl;
 
@@ -735,8 +740,8 @@ static void samples_dump(const char *filename)
 			 *  raw sample count by scaling by 1.0 (i.e. no scaling)
 			 */
 			if (sdi) {
-				duration = duration_round((opt_flags & OPT_SAMPLE_COUNT) ? 1.0 : sdi->time_delta);
-				fprintf(fp, ",%f", duration == 0.0 ? 0.0 :
+				double duration = duration_round((opt_flags & OPT_SAMPLE_COUNT) ? 1.0 : sdi->time_delta);
+				fprintf(fp, ",%f", FLOAT_CMP(duration, 0.0) ? 0.0 :
 					(double)sdi->delta / duration);
 			} else
 				fprintf(fp, ",%f", 0.0);
@@ -751,7 +756,6 @@ static void samples_dump(const char *filename)
 		fprintf(fp, ",Min:");
 		for (i = 0; i < n; i++) {
 			double min = DBL_MAX;
-			sample_delta_list_t *sdl;
 
 			for (sdl = sample_delta_list; sdl; sdl = sdl->next) {
 				sample_delta_item_t *sdi =
@@ -759,7 +763,7 @@ static void samples_dump(const char *filename)
 
 				if (sdi) {
 					double duration = duration_round((opt_flags & OPT_SAMPLE_COUNT) ? 1.0 : sdi->time_delta);
-					double val = (duration == 0.0) ?
+					double val = FLOAT_CMP(duration, 0.0) ?
 						0.0 : sdi->delta / duration;
 					if (min > val)
 						min = val;
@@ -772,7 +776,6 @@ static void samples_dump(const char *filename)
 		fprintf(fp, ",Max:");
 		for (i = 0; i < n; i++) {
 			double max = DBL_MIN;
-			sample_delta_list_t *sdl;
 
 			for (sdl = sample_delta_list; sdl; sdl= sdl->next) {
 				sample_delta_item_t *sdi =
@@ -780,7 +783,7 @@ static void samples_dump(const char *filename)
 
 				if (sdi) {
 					double duration = duration_round((opt_flags & OPT_SAMPLE_COUNT) ? 1.0 : sdi->time_delta);
-					double val = (duration == 0.0) ?
+					double val = FLOAT_CMP(duration, 0.0) ?
 						0.0 : sdi->delta / duration;
 					if (max < val)
 						max = val;
@@ -804,14 +807,13 @@ static void samples_dump(const char *filename)
 			double average = (double)
 				sorted_timer_infos[i]->total / (double)count;
 			double sum = 0.0;
-			sample_delta_list_t *sdl;
 
 			for (sdl = sample_delta_list; sdl; sdl = sdl->next) {
 				sample_delta_item_t *sdi =
 					sample_find(sdl, sorted_timer_infos[i]);
 				if (sdi) {
 					double duration = duration_round((opt_flags & OPT_SAMPLE_COUNT) ? 1.0 : sdi->time_delta);
-					double diff = duration == 0.0 ? 0.0 :
+					double diff = FLOAT_CMP(duration, 0.0) ? 0.0 :
 						((double)sdi->delta - average) / duration;
 					diff = diff * diff;
 					sum += diff;
